@@ -41,8 +41,8 @@ def saveCmd(s):
     if save2File:
         cmdStrings += s + "\n"
 
-def getValueFromStr(s):
-    if s.startswith('0x') or s.startswith('0X'):
+def getValueFromStr(s, isHex=False):
+    if isHex or s.startswith('0x') or s.startswith('0X'):
         return int(s, 16)
     else:
         return int(s)
@@ -53,15 +53,15 @@ def headTrans(head, value):
         v = getValueFromStr(value)
         value = socket.inet_ntoa(struct.pack('>I', socket.htonl(v)))
     elif type == "B16":
-        v = getValueFromStr(value)
+        v = getValueFromStr(value, True)
         v = struct.unpack('H', struct.pack('>H', v))
         value = "%x" % v
     elif type == "B32":
-        v = getValueFromStr(value)
+        v = getValueFromStr(value, True)
         v = struct.unpack('I', struct.pack('>I', v))
         value = "%x" % v
     elif type == "B64":
-        v = getValueFromStr(value)
+        v = getValueFromStr(value, True)
         v = struct.unpack('Q', struct.pack('>Q', v))
         value = "%x" % v
     elif type == "b16":
@@ -429,7 +429,10 @@ class ftrace(object):
         self._single = True
         self._start()
         pipe = self.baseDir + "/tracing/instances/surftrace/trace_pipe"
-        self.pipe = CasyncPipe(pipe, self.procLine)
+        if hasattr(self, "_cbOrig"):
+            self.pipe = CasyncPipe(pipe, self._cbOrig)
+        else:
+            self.pipe = CasyncPipe(pipe, self.procLine)
         saveCmd("cat %s" % pipe)
 
     def signalHandler(self, signalNumber, frame):
@@ -1031,8 +1034,8 @@ class surftrace(ftrace):
         self._stack = stack
         if isinstance(args, list):
             self._reSurfComm = re.compile(r"[a-zA-z][a-zA-z0-9_]*=\$comm")
-            self._reSurfProbe = re.compile(r"[a-zA-z][a-zA-z0-9_]*=[SUX]?(@\(struct .*\*\)(l[234]|)|\!\(.*\)|)%")
-            self._reSurfRet = re.compile(r"[a-zA-z][a-zA-z0-9_]*=[SUX]?(@\(struct .*\*\)(l[234]|)|\!\(.*\)|)\$retval")
+            self._reSurfProbe = re.compile(r"[a-zA-z][a-zA-z0-9_]*=[SUX]?(@\(struct .*\*\)(l[234]|)|\!\(.*\)|)(%|@|\$stack)")
+            self._reSurfRet = re.compile(r"[a-zA-z][a-zA-z0-9_]*=[SUX]?(@\(struct .*\*\)(l[234]|)|\!\(.*\)|)(\$retval|\$stack|@|%)")
             self._reLayer = re.compile(r"l[234]")
             self._reBrackets = re.compile(r"(?<=\().+?(?=\))")
             self._reSquareBrackets = re.compile(r"(?<=\[).+?(?=\])")
@@ -1046,24 +1049,17 @@ class surftrace(ftrace):
             self._strFxpr = ""
             self.__format = 'u'
             self._func = None
+            self._argSym = ""
 
         self._cb = cb
-        self._cbOrig = cbOrig
+        if cbOrig is not None:
+            self._cbOrig = cbOrig
 
         if cbShow: self._cbShow = cbShow
         else: self._cbShow = self._showFxpr
 
     def _getArchitecture(self):
-        lines = self._c.cmd('lscpu').split('\n')
-        for line in lines:
-            if line.startswith("Architecture"):
-                arch = line.split(":", 1)[1].strip()
-                if arch.startswith("arm"):
-                    return "arm"
-                if arch.startswith('x86'):
-                    return "x86"
-                return arch
-        return "Unkown"
+        return self._c.cmd('uname -m').strip()
 
     def _clearProbes(self):
         if self._show:
@@ -1177,7 +1173,7 @@ class surftrace(ftrace):
     
     def _splitXpr(self, xpr):
         for i, c in enumerate(xpr):
-            if c in ('.', '-'):
+            if c == '.' or (c == '-' and xpr[i + 1] == '>'):
                 return xpr[:i], xpr[i:]
         return xpr, ''
     
@@ -1272,9 +1268,10 @@ class surftrace(ftrace):
             raise DbException('db get %s return %s' % (func, res['log']))
         return res
 
-    def __getExprArgi(self, e, inFlag):
+    def __getExprArgi(self, e, inFlag=False):
         # expression: a=@(struct iphdr *)l4%1->saddr uesrs=!(struct task_struct *)%0->mm->mm_users
         # e is already checked at self.__checkBegExpr
+        argModeD = {'%': "mReg", '@': "mAddr", "$": "mVar"}
         var, expr = e.split("=", 1)
         self.__checkVar(var)
 
@@ -1282,28 +1279,43 @@ class surftrace(ftrace):
         if expr[0] in ('S', 'U', 'X'):
             expr = expr[1:]
             showType = expr[0]
-        if self._res['type'] == 'p':
-            types, xpr = expr.split('%', 1)
-            reg, xpr = self._splitXpr(xpr)
+
+        argMode = "None"
+        for k in argModeD.keys():
+            if k in expr:
+                self._argSym = k
+                argMode = argModeD[k]
+                types, xpr = expr.split(k, 1)
+        if argMode not in argModeD.values():
+            raise ExprException("bad arg mode for expr %s, mode: %s" % (expr, argMode))
+
+        reg, xpr = self._splitXpr(xpr)
+        if argMode == 'mReg':
             if reg.isdigit():
                 argi = int(reg)
             else:
                 argi = regIndex(reg, self._arch)
-            if types == '':
-                if inFlag:
-                    argt = 'u64'
-                else:
-                    argt = self._func['args'][argi]
-            else:
-                argt = types
             regArch = transReg(argi, self._arch)
         else:
-            types, xpr = expr.split('$retval')
-            if types == '':
+            argi = 0
+            regArch = reg
+
+        if not inFlag and argi > len(self._func['args']):
+            raise ExprException("argi num %d, which is larger than func args number %d." % (argi, len(self._func['args'])))
+
+        if types == '':
+            if argMode != "mReg":
+                argt = ''
+            elif inFlag:   # check mReg condition.
+                argt = 'u64'
+            elif self._res['type'] == 'p':
+                argt = self._func['args'][argi]
+            elif self._res['type'] == 'r':
                 argt = self._func['ret']
             else:
-                argt = types
-            regArch = '$retval'
+                raise ExprException("can not get arg type for %s" % expr)
+        else:
+            argt = types
         return showType, regArch, argt, xpr
     
     def _splitPr(self, argt, prs):
@@ -1459,12 +1471,11 @@ class surftrace(ftrace):
                 v = self.__netParse(sType, v)
         return sType, v
     
-    def _cellCheck(self, cells, reg):
-        if reg == "$retval":
-            self._strFxpr = reg
-        else:
-            self._strFxpr = "%" + reg
-            
+    def _cellCheck(self, cells, reg, getCell=False):
+        self._strFxpr = self._argSym + reg
+        if cells[0] == '':
+            return {"type": ''}
+
         i = 0; end = len(cells); lastCell = None
         sMem = ""; origType = sType = "unkown"; tStruct = None; origMode = '->'
 
@@ -1513,7 +1524,8 @@ class surftrace(ftrace):
                 cells[i + 2] = tMem
                 lastCell = {"type": tMem}
             i += 2
-        self._fxprAddSuffix(lastCell)
+        if not getCell:
+            self._fxprAddSuffix(lastCell)
         return lastCell
 
     def _checkExpr(self, e, inFlag):
@@ -1738,11 +1750,9 @@ class surftrace(ftrace):
 
         del self._parser
         if not self._show:
-            if self._cbOrig:
-                self.pipe.newCb(self._cbOrig)
             if self._cb is None:
                 self._cb = self._cbLine
-            super(surftrace, self).start()
+        super(surftrace, self).start()
 
     def stop(self):
         self._clearProbes()
@@ -1792,7 +1802,7 @@ def main():
 
     arch = ""
     if args.arch:
-        if arch not in ('x86', 'x86_64', 'aarch64'):
+        if args.arch not in ('x86', 'x86_64', 'aarch64'):
             raise InvalidArgsException('not support architecture %s' % args.arch)
         arch = args.arch
     localParser = setupParser(args.mode, args.db, args.rip, args.gdb, args.vmlinux, arch)
